//
// Class: documentPageCache
//
// Cache that holds a number of pages in a document.
// Part of KDVI- A previewer for TeX DVI files.
//
// (C) 2004 Stefan Kebekus. Distributed under the GPL.


#include <kdebug.h>
#include <qapplication.h>

#include "documentPageCache.h"
#include "documentRenderer.h"
#include "kvsprefs.h"
#include "renderedDocumentPagePixmap.h"


//#define documentPageCache_DEBUG


DocumentPageCache::DocumentPageCache()
{
  currentMemory = 0;
  // 32 MB cache memory.
  maxMemory = 2*16777216;
  LRUCache.setAutoDelete(false);
  recycleBin.setAutoDelete(false);

  resolutionInDPI = 0.0;
  Length w,h;
  w.setLength_in_mm(200);
  h.setLength_in_mm(300);
  userPreferredSize.setPageSize(w,h);
  useDocumentSpecifiedSize = true;
}


DocumentPageCache::~DocumentPageCache()
{
  LRUCache.setAutoDelete(true);
  recycleBin.setAutoDelete(true);
}


void DocumentPageCache::setRenderer(DocumentRenderer *_renderer)
{
  clear();
  renderer = _renderer;
}


SimplePageSize DocumentPageCache::sizeOfPage(PageNumber page) const
{
  // Paranoid safety checks
  if (!page.isValid()) {
    kdError(1223) << "DocumentPageCache::sizeOfPage( " <<  page << ") called with invalid page number." << endl;
    return SimplePageSize();
  }
  if (renderer.isNull()) {
    kdError(1223) << "DocumentPageCache::sizeOfPage( " <<  page << ") called when no renderer was set." << endl;
    return SimplePageSize();
  }

  SimplePageSize s = renderer->sizeOfPage(page);
  if (!useDocumentSpecifiedSize)
    s = userPreferredSize;

  if (!s.isValid())
  {
    // If the size is invalid use the size of the first Page in the document
    // as an estimate.
    s = renderer->sizeOfPage(1);
    if (!s.isValid())
      s = userPreferredSize;
  }

  return s;
}

void DocumentPageCache::setResolution(double res)
{
  if (res != resolutionInDPI)
  {
    resolutionInDPI = res;
    clear();
  }
}

QSize DocumentPageCache::sizeOfPageInPixel(PageNumber pg) const
{
  // Paranoid safety checks
  if (renderer.isNull()) {
    kdError(1223) << "DocumentPageCache::sizeOfPageInPixel( " << pg << " ) called but no renderer was set" << endl;
    return QSize();
  }
  if (!pg.isValid()) {
    kdError(1223) << "DocumentPageCache::sizeOfPageInPixel( " << pg << " ) called with invalid argument" << endl;
    return QSize();
  }

  SimplePageSize ps = sizeOfPage(pg);
  if (ps.isValid())
    return ps.sizeInPixel(resolutionInDPI);
  return userPreferredSize.sizeInPixel(resolutionInDPI); 
}


bool DocumentPageCache::isPageCached(PageNumber pageNumber, QSize size)
{
  // Paranoid checks
  if (renderer.isNull()) {
    kdError(1223) << "DocumentPageCache::isPageCached(..) called but no renderer was set" << endl;
    return false;
  }
  if (!pageNumber.isValid()) {
    kdError(1223) << "DocumentPageCache::isPageCached( " << pageNumber << " ) called, with invalid argument." << endl;
    return false;
  }
  if (renderer->totalPages() < pageNumber) {
    kdError(1223) << "DocumentPageCache::isPageCached( " << pageNumber
                  << " ) called but document contains only " << renderer->totalPages() << " pages." << endl;
    return false;
  }

  // Check if the page that we are looking for is in the cache
  RenderedDocumentPagePixmap* page;
  for (page = LRUCache.first(); page; page = LRUCache.next()) {
    if (page->getPageNumber() == pageNumber) {
      if (page->size() == size)
        return true;
      else {
        kdError(1223) << "The requested page " << page->getPageNumber() << " was found, but the sizes"
                      << " did not match. " << page->size() << "!=" << size << endl;
        return false;
      }
    }
  }

  // Page pageNumber is not in the Cache
  return false;
}

bool DocumentPageCache::isPageCached(PageNumber pageNumber)
{
  // Paranoid checks
  if (renderer.isNull()) {
    kdError(1223) << "DocumentPageCache::isPageCached(..) called but no renderer was set" << endl;
    return false;
  }
  if (!pageNumber.isValid()) {
    kdError(1223) << "DocumentPageCache::isPageCached( " << pageNumber << " ) called, with invalid argument." << endl;
    return false;
  }
  if (renderer->totalPages() < pageNumber) {
    kdError(1223) << "DocumentPageCache::isPageCached( " << pageNumber
                  << " ) called but document contains only " << renderer->totalPages() << " pages." << endl;
    return false;
  }

  // Check if the page that we are looking for is in the cache
  RenderedDocumentPagePixmap* page;
  for (page = LRUCache.first(); page; page = LRUCache.next()) {
    if (page->getPageNumber() == pageNumber)
      return true;
  }

  // Page pageNumber is not in the Cache
  return false;
}

RenderedDocumentPagePixmap *DocumentPageCache::getPage(PageNumber pageNr)
{
#ifdef DocumentPageCache_DEBUG
  kdDebug(1223) << "DocumentPageCache::getPage( pageNr=" << pageNr << " )" << endl;
#endif

  // Paranoid checks
  if (renderer.isNull()) {
    kdError(1223) << "DocumentPageCache::getPage(..) called but no renderer was set" << endl;
    return 0;
  }
  if (!pageNr.isValid()) {
    kdError(1223) << "DocumentPageCache::getPage( " << pageNr << " ) called, with invalid argument." << endl;
    return 0;
  }
  if (renderer->totalPages() < pageNr) {
    kdError(1223) << "DocumentPageCache::getPage( " << pageNr << " ) called but document contains only " << renderer->totalPages() << " pages." << endl;
    return 0;
  }

  // First check if the page that we are looking for is in the cache
  RenderedDocumentPagePixmap *page;
  for ( page = LRUCache.first(); page; page = LRUCache.next() )
    if ((page->getPageNumber() == pageNr) && (page->isEmpty == false)) {
      // Page found! Move the page to the front of the list, and
      // return it.
      LRUCache.remove();
      LRUCache.prepend(page);
      return page;
    }

  // The page was not found in the cache, so we have to make a new
  // page and add this to the cache. We have several options where
  // this page is supposed to come from.

  // If the cache already contains the maximum number of elements, we
  // use the last recently element, move it to the front of the list
  // and use that.
  if (currentMemory >= maxMemory) {
    page = LRUCache.getLast();
    LRUCache.removeLast();
    // Memory is freed.
    currentMemory -= page->memory();
    page->clear();
  }

  // If we haven't found a structure in the cache, perhaps there is a
  // page structure waiting in the recycleBin that we could use?
  if ((page == 0) && (recycleBin.isEmpty() == false)) {
    page = recycleBin.first();
    recycleBin.removeFirst();
  }
  
  // If we haven't found a structure, we allocate a new one
  if (page == 0)
    page = createDocumentPagePixmap();
  
  // If even that failed, issue an error message and quit.
  if (page == 0) {
    kdError(1223) << "DocumentPageCache::getPage(..) cannot allocate DocumentPage structure" << endl;
    return 0;
  }
  
  // Now 'page' contains a point to a page structure that we can
  // use. Add the page as the first element to the cache, and apply
  // the renderer to the page
  LRUCache.prepend(page);
  page->setPageNumber(pageNr);
  if (!renderer.isNull()) {
    if (resolutionInDPI > 0.0) {
      page->resize(sizeOfPageInPixel(pageNr));
      currentMemory += page->memory();
      QApplication::setOverrideCursor( waitCursor );
      renderer->drawPage(resolutionInDPI, page);
      QApplication::restoreOverrideCursor();
    }
    else
      kdError(1223) << "DocumentPageCache::getPage() called, but no resolution or negative resolution was set" << endl;
  }
  
  //kdDebug(1223) << "page cache (currentMemory, maxMemory) = (" << currentMemory << ", " << maxMemory << ")" << endl;

  //kdDebug() << "documentPageChache: draw page" << pageNr << endl;

  return page;
}


RenderedDocumentPagePixmap* DocumentPageCache::createDocumentPagePixmap()
{
 return new RenderedDocumentPagePixmap();
}


void DocumentPageCache::clear()
{
  RenderedDocumentPagePixmap *page;

  while( (page = LRUCache.first()) != 0 ) {
    LRUCache.removeFirst();
    page->clear();
    recycleBin.prepend(page);
  }
  currentMemory = 0;
}


void DocumentPageCache::setUserPreferredSize(SimplePageSize s)
{
  bool sizeChanged = !userPreferredSize.isNearlyEqual(s);
  userPreferredSize = s;

  if (sizeChanged)
    emit(paperSizeChanged());
}


void DocumentPageCache::setUseDocumentSpecifiedSize(bool b)
{
  bool valChanged = (useDocumentSpecifiedSize == b);

  useDocumentSpecifiedSize = b;
  if (valChanged)
    emit(paperSizeChanged());
}


QPixmap DocumentPageCache::createThumbnail(PageNumber pageNr, int width)
{
  // Paranoid checks
  if (renderer.isNull()) {
    kdError(1223) << "DocumentPageCache::createThumbnail(..) called but no renderer was set" << endl;
    thumbnailPage.resize(0,0);
    return thumbnailPage;
  }
  if (renderer->totalPages() < pageNr) {
    kdError(1223) << "DocumentPageCache::createThumbnail( " << pageNr << ", width ) called but document contains only " << renderer->totalPages() << " pages." << endl;
    thumbnailPage.resize(0,0);
    return thumbnailPage;
  }
  if (!pageNr.isValid()) {
    kdError(1223) << "DocumentPageCache::createThumbnail(..) called for page with invalid page specification" << endl;
    thumbnailPage.resize(0,0);
    return thumbnailPage;
  }
  if (!sizeOfPage().isValid()) {
    kdError(1223) << "DocumentPageCache::createThumbnail(..) called for page with invalid size" << endl;
    thumbnailPage.resize(0,0);
    return thumbnailPage;
  }

  thumbnailPage.setPageNumber(pageNr);
  thumbnailPage.resize(width, (int)(width/sizeOfPage(pageNr).aspectRatio() + 0.5 ) );
  renderer->drawThumbnail((double)(width)/sizeOfPage(pageNr).width().getLength_in_inch(), &thumbnailPage);

  if (KVSPrefs::changeColors() && KVSPrefs::renderMode() != KVSPrefs::EnumRenderMode::Paper)
  {
    return thumbnailPage.accessiblePixmap();
  }
  else
  {
    return thumbnailPage;
  }
}

void DocumentPageCache::deselectText() 
{
  userSelection.clear();
  emit textSelected(false);
}

void DocumentPageCache::selectText(TextSelection selection) 
{
  userSelection = selection;
  emit textSelected(!userSelection.isEmpty());
}

#include "documentPageCache.moc"
