// -*- C++ -*-
//
// Class: documentPageCache
//
// Cache that holds a number of pages in a document.
// Part of KDVI- A previewer for TeX DVI files.
//
// (C) 2004 Stefan Kebekus. Distributed under the GPL.


#ifndef _documentpagecache_h_
#define _documentpagecache_h_

#include <qobject.h>
#include <qguardedptr.h>
#include <qpixmap.h>
#include <qrect.h>
#include <qstring.h>
#include <qvaluevector.h>

#include "renderedDocumentPagePixmap.h"
#include "pageNumber.h"
#include "pageSize.h"
#include "selection.h"

class RenderedDocumentPage;
class DocumentRenderer;


class DocumentPageCache: public QObject
{
 Q_OBJECT

 public:
  DocumentPageCache();
  virtual ~DocumentPageCache();

  /** This method is used to name the DocumentRenderer, that the
      documentPageCache uses to render the page. The renderer can be
      used any time (e.g., triggered by an internal timer event), and
      must not be deleted before either the DocumentRenderer is
      deleted, or another renderer has been set. */
  void     setRenderer(DocumentRenderer *_renderer);

  void     setResolution(double res);
  double   getResolution() const {return resolutionInDPI;}

  TextSelection selectedText() { return userSelection; }

  void deselectText();
  void selectText(TextSelection selection);

  /** Returns the size of page 'page'. If the document does not
      specify a size (which happens, e.g., for some DVI-files), then
      the userPreferredSize is returned. */
  SimplePageSize sizeOfPage(PageNumber page = 1) const;

  /** Returns the size of page 'page', in pixels, using the resolution
      set with setResolution(). If the document does not specify a
      size (which happens, e.g., for some DVI-files), the
      userPreferredSize is used. */
  QSize sizeOfPageInPixel(PageNumber page) const;

  /** Returns a pointer to a documentPage structure, or 0, if the
      documentPage could not be generated for some reason (e.g.,
      because no renderer has been set). */
  RenderedDocumentPagePixmap *getPage(PageNumber pageNr);

  /** Checks if the given page is already in the cache. */
  bool isPageCached(PageNumber pageNumber, QSize size);

  /** Checks if the given page is already in the cache. Here we don't care about the size
      of the page. */
  bool isPageCached(PageNumber pageNumber);

  /** Returns a "width" pixels width thumbnail of the given page. This
      method returns an empty QPixmap if one of the arguments is
      invalid, or if the page cannot be rendered for any reason. */
  QPixmap createThumbnail(PageNumber pageNr, int width);

 signals:
  /** This signal is emitted when setUserPreferredSize() or
      setUseUserPreferredSize() is called, and the page size
      changes accordingly. */
  void paperSizeChanged();

  /** This signal is emitted when the text selection of the current
      document changes. The argument is false if no text is selected, true
      otherwise. */
  void textSelected(bool);

 public slots:
  /** Clears the contents of the cache. */
  void clear();

  /** Sets the userPreferredSize, which is explained below */
  void setUserPreferredSize(SimplePageSize t);
  void setUseDocumentSpecifiedSize(bool);

 protected:
  /** This function creates new RenderedDocumentPagePixmap objects.
      If a multipage implementation needs additional functionality overwrite
      this function to create objects of a suitable subclass of RenderedDocumentPagePixmap.
  */
  virtual RenderedDocumentPagePixmap* createDocumentPagePixmap();

 private:
  /** The maximum of memory used by the cache. (32MB)
      TODO: make this configurable, or detact an appropriate value at startup. */
  Q_UINT32 maxMemory;

  /** The amount of memory that is currently used by the pixmap cache. */
  Q_UINT32 currentMemory;

  /** This field contains resolution of the display device. In
      principle. In fact, kviewshell implements zooming by calling the
      setResolution()-method with values that are not exactly the
      resolution of the display, but multiplied with the zoom
      factor. Bottom line: the documentRenderer should act as if this
      field indeed contains resolution of the display device. Whene a
      documentRenderer is constructed, this field is set to the actual
      resolution to give a reasonable default value. */
  double              resolutionInDPI;

  SimplePageSize      userPreferredSize;
  bool                useDocumentSpecifiedSize;

  TextSelection userSelection;

  /** This list holds the cache. The least recently accessed page is
      always the last. */
  QPtrList<RenderedDocumentPagePixmap> LRUCache;

  /** If a page is removed from the LRUCache, it is not deleted, but
      clear()ed, and stored in this recycleBin. The page can then
      later be re-used. This makes for a substantial preformance
      gain. */
  QPtrList<RenderedDocumentPagePixmap> recycleBin;

  /** This is a dummy documentPage structure which is used internally
      by the 'createThumbnail' method. */
  RenderedDocumentPagePixmap thumbnailPage;

  QGuardedPtr<DocumentRenderer> renderer;
};


#endif
